<?php

namespace Drupal\googlenews_sitemap\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\Cache;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;

/**
 * Controller for generating Google News sitemap.
 */
class GoogleNewsSitemapController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The cache backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a GoogleNewsSitemapController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    ConfigFactoryInterface $config_factory,
    CacheBackendInterface $cache,
    RendererInterface $renderer,
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->cache = $cache;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('cache.default'),
      $container->get('renderer'),
    );
  }

  /**
   * Generates the Google News sitemap.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response containing the XML sitemap.
   */
  public function generateSitemap() {
    // Execute the sitemap generation in a render context.
    $context = new RenderContext();
    $xml_content = $this->renderer->executeInRenderContext($context, function () {
      // Get module configuration.
      $config = $this->configFactory->get('googlenews_sitemap.settings');

      // Initialize cache ID and domain variables.
      $cid = 'googlenews_sitemap:xml';
      $domain = NULL;

      // Check if domain module is enabled and get domain context if it is.
      if (\Drupal::moduleHandler()->moduleExists('domain')) {
        $domain = \Drupal::service('domain.negotiator')->getActiveId();
        $cid .= ':' . $domain;
      }

      // Check if we have a cached version.
      if ($cache = $this->cache->get($cid)) {
        return $cache->data;
      }

      // Get content types to include in the sitemap.
      $content_types = $config->get('content_types') ?: [];

      // Get publication name.
      $publication_name = $config->get('publication_name') ?: $this->config('system.site')->get('name');

      // Get publication language.
      $publication_language = $config->get('publication_language') ?: 'en';

      // Get the maximum age of news items in hours.
      $max_age_hours = $config->get('max_age_hours') ?: 48;

      // Calculate the cutoff time for news items.
      $cutoff_time = strtotime('-' . $max_age_hours . ' hours');

      // Start building the XML.
      $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" xmlns:news="http://www.google.com/schemas/sitemap-news/0.9"></urlset>');

      // Query for recent news content.
      $query = $this->entityTypeManager->getStorage('node')->getQuery()
        ->condition('status', 1)
        ->accessCheck(TRUE)
        ->condition('created', $cutoff_time, '>=');

      // Add domain access condition only if domain module is enabled.
      if ($domain !== NULL) {
        $query->condition('field_domain_access', $domain);
      }

      // Filter by content types if specified.
      if (!empty($content_types)) {
        $query->condition('type', $content_types, 'IN');
      }

      // Sort by creation date, newest first.
      $query->sort('created', 'DESC');

      // Execute the query.
      $nids = $query->execute();

      // Load the nodes.
      $nodes = $this->entityTypeManager->getStorage('node')->loadMultiple($nids);

      // Add each node to the sitemap.
      foreach ($nodes as $node) {
        $url = $xml->addChild('url');
        $url->addChild('loc', $node->toUrl('canonical', ['absolute' => TRUE])->toString());

        $news = $url->addChild('news:news', '', 'http://www.google.com/schemas/sitemap-news/0.9');

        $publication = $news->addChild('news:publication', '', 'http://www.google.com/schemas/sitemap-news/0.9');
        $publication->addChild('news:name', htmlspecialchars($publication_name), 'http://www.google.com/schemas/sitemap-news/0.9');
        $publication->addChild('news:language', $publication_language, 'http://www.google.com/schemas/sitemap-news/0.9');

        $date = new DrupalDateTime('@' . $node->created->value);
        $news->addChild('news:publication_date', $date->format('Y-m-d\TH:i:sP'), 'http://www.google.com/schemas/sitemap-news/0.9');

        $news->addChild('news:title', htmlspecialchars($node->label()), 'http://www.google.com/schemas/sitemap-news/0.9');
      }

      $xml_content = $xml->asXML();

      // Cache the XML output.
      $this->cache->set(
        $cid,
        $xml_content,
        Cache::PERMANENT,
        ['node_list', 'googlenews_sitemap']
      );

      return $xml_content;
    });

    // Create the response.
    $response = new Response($xml_content);
    $response->headers->set('Content-Type', 'application/xml');
    $response->headers->set('Cache-Control', 'public, max-age=3600');

    return $response;
  }

}
