<?php

namespace Drupal\googlenews_sitemap\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Google News Sitemap settings.
 */
class GoogleNewsSitemapSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a GoogleNewsSitemapSettingsForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'googlenews_sitemap_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['googlenews_sitemap.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('googlenews_sitemap.settings');

    $form['publication_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Publication Settings'),
      '#open' => TRUE,
    ];

    $form['publication_settings']['publication_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publication Name'),
      '#description' => $this->t('The name of your publication as it should appear in Google News.'),
      '#default_value' => $config->get('publication_name') ?: $this->config('system.site')->get('name'),
      '#required' => TRUE,
    ];

    $form['publication_settings']['publication_language'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publication Language'),
      '#description' => $this->t('The language code of your publication (e.g., en, fr, de).'),
      '#default_value' => $config->get('publication_language') ?: 'en',
      '#required' => TRUE,
      '#size' => 10,
    ];

    $form['content_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Content Settings'),
      '#open' => TRUE,
    ];

    // Get all content types.
    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();
    $options = [];
    foreach ($content_types as $type) {
      $options[$type->id()] = $type->label();
    }

    $form['content_settings']['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content Types to Include'),
      '#description' => $this->t('Select which content types should be included in the Google News sitemap.'),
      '#options' => $options,
      '#default_value' => $config->get('content_types') ?: [],
    ];

    $form['content_settings']['max_age_hours'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum Age (hours)'),
      '#description' => $this->t('Google News only accepts content published within the last 48 hours. You can set a lower value if needed.'),
      '#default_value' => $config->get('max_age_hours') ?: 48,
      '#min' => 1,
      '#max' => 48,
      '#required' => TRUE,
    ];

    $form['sitemap_info'] = [
      '#type' => 'markup',
      '#markup' => $this->t('<p>Your Google News sitemap will be available at: <strong>/news-sitemap.xml</strong></p>'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('googlenews_sitemap.settings')
      ->set('publication_name', $form_state->getValue('publication_name'))
      ->set('publication_language', $form_state->getValue('publication_language'))
      ->set('content_types', array_filter($form_state->getValue('content_types')))
      ->set('max_age_hours', $form_state->getValue('max_age_hours'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
